//*****************************************************************************
//              \ /
// No Copyright (c) 2003
//              / \
// No rights reserved. This material contains published,
// non-copyrighted work, which includes non-confidential and
// non-proprietary information of Kurt Santel.
//
// Source: Arena.cpp
//
// Description: Server-side Arena class which describes the arena where
//              the robots will live
//
//*****************************************************************************

#include "arena.h"
#include <time.h>

Arena::Arena(XType width, YType height)
: m_width( width )
, m_height( height )
{
    // Resize the vectors
    m_Squares.resize( m_width );

    for( XType i=0; i<m_width; ++i )
    {
        m_Squares[i].resize( m_height );
    }

    // Create the Squares
    for( i=0; i<m_width; ++i )
    {
        for( YType j=0; j<m_height; ++j )
        {
            if( (0 == i) || (0 == j) || (i==m_width-1) || (j==m_height-1) )
            {
                m_Squares[i][j] = Square( Square::Border );
            }
            else
            {
                m_Squares[i][j] = Square( Square::Empty );
            }
        }
    }

    // Assign the neighbours
    for( i=0; i<m_width; ++i )
    {
        for( YType j=0; j<m_height; ++j )
        {
            if( (0 == i) && (0 == j) )                          // South-West corner
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( NULL );
                m_Squares[i][j].setWest( NULL );
            }
            else if( (0 == i) && (m_height-1 == j) )            // North-West corner
            {
                m_Squares[i][j].setNorth( NULL );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( NULL );
            }
            else if( (m_width-1 == i) && (0 == j) )             // South-East corner
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( NULL );
                m_Squares[i][j].setSouth( NULL );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
            else if( (m_width-1 == i) && (m_height-1 == j) )    // North-East corner
            {
                m_Squares[i][j].setNorth( NULL );
                m_Squares[i][j].setEast( NULL );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
            else if( 0 == i )                                   // West border
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( NULL );
            }
            else if( 0 == j )                                   // South border
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( NULL );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
            else if( m_width-1 == i )                           // East border
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( NULL );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
            else if( m_height-1 == j )                          // North border
            {
                m_Squares[i][j].setNorth( NULL );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
            else
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
        }
    }

    ::srand( (unsigned)time( NULL ) );

    // Assign random Squares the property food
    Square * square = NULL;

    for( int nr=0; nr<valMaxFoodSquares; ++nr )
    {
        getRandomSquare( &square );

        if( square && Square::Empty == square->getType() )
        {
            square->setType( Square::Food );
        }
    }

    // Assign random Squares the property life
    for( nr=0; nr<valMaxLifeSquares; ++nr )
    {
        getRandomSquare( &square );

        if( square && Square::Empty == square->getType() )
        {
            square->setType( Square::Life );
        }
    }
}

Arena::~Arena(void)
{
    for( XType i=0; i<m_width; ++i )
    {
        m_Squares[i].clear();
    }

    m_Squares.clear();
}

Square * Arena::getSquare(XType x, YType y)
{
    if( (x >= 0) && (x < m_width) && (y >= 0) && (y < m_height) )
    {
        return( &m_Squares[x][y] );
    }

    return( NULL );
}

void Arena::getRandomSquare(Square * * square)
{
    if( (2 >= m_width) || (2 >= m_height) )
    {
        *square = NULL;
        return;
    }

    PositionType position;

    position.first = (XType)(::rand() % (m_width - 2) + 1);
    position.second = (YType)(::rand() % (m_height - 2) + 1);

    *square = &m_Squares[position.first][position.second];
}
