#include "d3dbox.h"

D3DBox::D3DBox(void)
: m_pBoxMesh( NULL )
, m_pd3dDevice( NULL )
, m_CurrentTime( 0 )
, m_LastTime( 0 )
, m_red( 1.0f )
, m_green( 1.0f )
, m_blue( 0.0f )
{
}

D3DBox::~D3DBox(void)
{
    if( NULL != m_pBoxMesh )
    {
        m_pBoxMesh->Release();
    }
}

bool D3DBox::create(IDirect3DDevice8 * pd3dDevice)
{
    if( NULL == pd3dDevice )
    {
        return( false );
    }

    if( NULL != m_pBoxMesh )
    {
        m_pBoxMesh->Release();
        m_pBoxMesh = NULL;
    }
    
    m_pd3dDevice = pd3dDevice;

    LPD3DXMESH pTempBoxMesh = NULL;

    //
    // We'll use the D3DXCreateBox utility function to create a simple 
    // box meshes to experiment with.
    //

    if( FAILED( D3DXCreateBox( m_pd3dDevice, 1.0f, 1.0f, 1.0f, &pTempBoxMesh, NULL ) ) )
    {
        return( false );
    }

    //
    // Unfortunately, the D3DXCreateBox utility function creates a mesh
    // with no color, so we'll need to make a clone of the original meshes
    // using a FVF code that does include color.
    //
    // Once that's been done, we'll need to set the color values.
    //

    //
    // Clone the original mesh and make it yellow...
    //

    LPDIRECT3DVERTEXBUFFER8 pTempVertexBuffer;

    pTempBoxMesh->CloneMeshFVF( 0, D3DFVF_XYZ | D3DFVF_DIFFUSE, m_pd3dDevice, &m_pBoxMesh );

    if( SUCCEEDED( m_pBoxMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
    {
        int nNumVerts = m_pBoxMesh->GetNumVertices();
        Vertex *pVertices = NULL;

        pTempVertexBuffer->Lock( 0, 0, (BYTE**)&pVertices, 0 );

        for( int i = 0; i < nNumVerts; ++i )
        {
            pVertices[i].diffuse = D3DCOLOR_COLORVALUE( m_red, m_green, m_blue, 1.0 );
        }

        pTempVertexBuffer->Unlock();

        pTempVertexBuffer->Release();
    }

    //
    // The temp mesh is no longer needed...
    //

    pTempBoxMesh->Release();

    return( true );
}

bool D3DBox::setColor(float r, float g, float b)
{
    m_red = r;
    m_green = g;
    m_blue = b;

    if( !create( m_pd3dDevice ) )
    {
        return( false );
    }

    return( true );
}

void D3DBox::move(float x, float y, float z)
{
    D3DXMatrixTranslation( &m_mMove, x, y, z );
}

void D3DBox::scale(float x, float y, float z)
{
    D3DXMatrixScaling( &m_mScale, x, y, z );
}

bool D3DBox::draw(void)
{
    if( NULL == m_pd3dDevice )
    {
        return( false );
    }

//    static float fSpin = 0.0f;
//    static float multiplier = 4.0f;

//    m_CurrentTime = ::timeGetTime();
//    float fElpasedTime = (float)((m_CurrentTime - m_LastTime) * 0.001);
//    m_LastTime = m_CurrentTime;

//    fSpin += multiplier * (fElpasedTime * 10.0f);

    //
    // The box is easy because the mesh for it is initially created centered  
    // at origin. All we have to do is spin it by rotating it about the Y axis.
    //

//    D3DXMATRIX mSpinRotation;
    D3DXMATRIX mSunMatrix;

//    D3DXMatrixRotationY( &mSpinRotation, D3DXToRadian( fSpin ) );

    // Now, concatenate them together...

    mSunMatrix = m_mScale *         // 1. Uniformly scale the box in size
//                 mSpinRotation *    // 2. and then spin it on its axis
                 m_mMove;           // 3. and finally move the box.

    m_pd3dDevice->SetTransform( D3DTS_WORLD, &mSunMatrix );

    if( m_pBoxMesh )
    {
        if( SUCCEEDED( m_pBoxMesh->DrawSubset( 0 ) ) )
        {
            return( true );
        }
    }

    return( false );
}
