//*****************************************************************************
//              \ /
// No Copyright (c) 2003
//              / \
// No rights reserved. This material contains published,
// non-copyrighted work, which includes non-confidential and
// non-proprietary information of Kurt Santel.
//
// Source: Robot.h
//
// Description: Server-side component that keeps track of the properties
//              of a Robot
//
//*****************************************************************************

#pragma once

#include "../CodeWarCommon/CodeWarTypes.h"

class Square;
struct skipTurnCommand;
struct restCommand;
struct moveForwardCommand;
struct moveBackwardCommand;
struct turnLeftCommand;
struct turnRightCommand;
struct eatCommand;
struct fightCommand;

// The properties of the robots that are registered will be kept using
// this class.  Since every SlaveThread handles one robot, every SlaveThread
// will have one Robot instance.
class Robot
{
public:
    Robot(IDType id=-1, AttackType a=-1, DefenseType d=-1);
    ~Robot(void);

    // Must be called before a doTurn() function call
    bool checkPreConditions(void);

    // Adjust the properties
    bool doTurn(skipTurnCommand const & skip);
    bool doTurn(restCommand const & rest);
    bool doTurn(moveForwardCommand const & mf);
    bool doTurn(moveBackwardCommand const & mb);
    bool doTurn(turnLeftCommand const & tl);
    bool doTurn(turnRightCommand const & tr);
    bool doTurn(eatCommand const & eat);
    bool doTurn(fightCommand const & fight);

    // Must be called after a doTurn() function call
    bool checkPostConditions(void);

public:
    // Control properties
    bool m_hasTurn;             // The server is expecting a command
    bool m_hadTurn;             // The server has received a command
    bool m_registered;

    // Personal properties

    //*****************************************************************************
    // ID is not really needed.
    IDType m_ID;                // The ID is assigned when the robot is registered
    //*****************************************************************************

    AttackType m_attack;        // The attack strenght of the robot
    DefenseType m_defense;      // The defense strenght of the robot
    Square * m_position;        // The position in the arena
    HeadingType m_heading;      // North, East, South or West
    FoodType m_food;            // Hunger status; 0 == starvation starts
    LifeType m_life;            // Life status; 0 == dead meat (or metal :o)

    // Event properties
    bool m_foodFound;           // I'm standing on food!
    bool m_dead;                // Am I dead?
    bool m_wallHit;             // Ouch, that must hurt!
    bool m_robotInSight;        // There is a robot one Square further
                                // in the direction you are heading.
    bool m_attacked;            // I'm under attack! Help!
};
