//*****************************************************************************
//              \ /
// No Copyright (c) 2003
//              / \
// No rights reserved. This material contains published,
// non-copyrighted work, which includes non-confidential and
// non-proprietary information of Kurt Santel.
//
// Source: Square.h
//
// Description: Server-side Square class which describes the unit out of which
//              the Arena exists
//
//*****************************************************************************

#pragma once

#include <list>
class Robot;

class Square
{
public:
    typedef enum SquareType
    {
        Empty,              // Nothing can be found in this Square
        Border,             // This Square is a border.  Look out, u might hit a wall ;-)
        Food,               // This Square contains food (unlimited for the moment)
//        Energy,             // This Square contains energy (unlimited for the moment)
        Life                // This Square contains life (unlimited for the moment)
    } SquareType;

    Square(SquareType type=Empty);
    ~Square(void);

    void setType(SquareType type) { m_Type = type; }
    SquareType getType(void) const { return( m_Type ); }

    void setNorth(Square * north) { m_North = north; }
    void setEast(Square * east)   { m_East = east; }
    void setSouth(Square * south) { m_South = south; }
    void setWest(Square * west)   { m_West = west; }

    Square * getNorth(void) const { return( m_North ); }
    Square * getEast(void)  const { return( m_East ); }
    Square * getSouth(void) const { return( m_South ); }
    Square * getWest(void)  const { return( m_West ); }

    // Every Square keeps track of the Robots that are on it
    void addRobot(Robot * robot);
    void removeRobot(Robot * robot);
    void getRobots(std::list<Robot *> & robots) const;

private:
    SquareType m_Type;

    // Neighbours, everybody needs good neighbours, with a little understanding, ...
    Square * m_North;
    Square * m_East;
    Square * m_South;
    Square * m_West;

    std::list<Robot *> m_Robots;
};
